#include <stdio.h>
#include <stdlib.h>

/* Ukljucujmo i zaglavlje sa definisanim tokenima */
#include "tokeni.h"

/* Gramatika za koju radimo sintaksnu analizu je:
 * Naredba        ­> While_naredba
 *                   | If_naredba
 *                   | Naredba_dodele
 *                   | Blok_naredba
 *                   | eps
 * While_naredba  ­> while (Uslov) do Naredba
 * If_naredba     ­> if (Uslov) then Naredba else Naredba
 * Naredba_dodele ­> id := broj
 * Blok_naredba   ­> begin Niz_naredbi end
 * Niz_naredbi    ­> Niz_naredbi ; Naredba
 *                   | Naredba
 * Uslov          ­> id relacijski_operator broj
 *
 * Da bi mogli da primenimo rekurzivni spust, moracemo da 
 * se oslobodimo leve rekurzije koju imamo u pravilu za Niz_naredbi
 *
 *
 * Umesto ovog pravila, dobijamo dva nova pravila, tako da ce
 * nova gramatika bez leve rekurzije izgledati
 * Naredba        ­> While_naredba
 *                   | If_naredba
 *                   | Naredba_dodele
 *                   | Blok_naredba
 *                   | eps
 * While_naredba  ­> while (Uslov) do Naredba
 * If_naredba     ­> if (Uslov) then Naredba else Naredba
 * Naredba_dodele ­> id := broj
 * Blok_naredba   ­> begin Niz_naredbi end
 * Niz_naredbi    ­> Naredba Niz_naredbi'
 * Niz_naredbi'   -> ; Naredba Niz_naredbi'
 *                   | eps
 * Uslov          ­> id relacijski_operator broj 
 * Za zadatu gramatiku treba odrediti skupove izbora.
 * A pre toga odredimo skupove FIRST i FOLLOW
 *                  FIRST                          FOLLOW
 * Naredba          while, if, id, begin, eps      eoi, else, ;, end
 * While_naredba    while                          eoi, else, ;, end
 * If_naredba       if                             eoi, else, ;, end
 * Naredba_dodele   id                             eoi, else, ;, end
 * Blok_naredba     begin                          eoi, else, ;, end
 * Niz_naredbi      while, if, id, begin, ;, eps   end
 * Niz_naredbi'     ;, eps                         end
 * Uslov            id                             )
 *
 * Skupovi izbora su sada sledeci:
 * Pravila                                                    Skupovi izbora
 * Naredba        ­> While_naredba                            while
 *                   | If_naredba                             if
 *                   | Naredba_dodele                         id
 *                   | Blok_naredba                           begin
 *                   | eps                                    eoi, else, ;, end
 * While_naredba  ­> while (Uslov) do Naredba                 while
 * If_naredba     ­> if (Uslov) then Naredba else Naredba     if
 * Naredba_dodele ­> id := broj                               id
 * Blok_naredba   ­> begin Niz_naredbi end                    begin
 * Niz_naredbi    ­> Naredba Niz_naredbi'                     while, if, id, begin, ;, end
 * Niz_naredbi'   -> ; Naredba Niz_naredbi'                   ;
 *                   | eps                                    end
 * Uslov          ­> id relacijski_operator broj              id
 */

/* Promenljiva koja je sluzila u fazi testiranja programa */
/* #define ISPIS 1 */



/* Naznacujemo da ce yylex biti deklarisana u nekoj drugoj datoteci */
extern int yylex ();

/* Preduvidni token */
int lookahead;


/* Pravimo makroe koji ce nam biti od koristi */
#define match(x) (lookahead==(x))
#define advance(x) (lookahead = yylex())

/* Funkcija za obradu gresaka */
void
error (char *message)
{
  fprintf (stderr, "%s\n", message);
  exit (1);
}


/* Deklarisemo rekurzivne funkcije koje odgovaraju neterminalima */
void Naredba ();
void While_naredba ();
void If_naredba ();
void Naredba_dodele ();
void Blok_naredba ();
void Niz_naredbi ();
void Niz_naredbiP ();
void Uslov ();

/* Definicije funkcija */
void
Naredba ()
{
  if (match (TOKEN_WHILE))
    {
#ifdef ISPIS
      printf ("Naredba -> While_naredba\n");
#endif
      While_naredba ();
    }
  else if (match (TOKEN_IF))
    {
#ifdef ISPIS
      printf ("Naredba -> If_naredba\n");
#endif
      If_naredba ();
    }
  else if (match (TOKEN_ID))
    {
#ifdef ISPIS
      printf ("Naredba -> Naredba_dodele\n");
#endif
      Naredba_dodele ();
    }
  else if (match (TOKEN_BEGIN))
    {
#ifdef ISPIS
      printf ("Naredba -> Blok_naredba\n");
#endif
      Blok_naredba ();
    }
  else if (match (TOKEN_EOI) || match (TOKEN_ELSE) || match (TOKEN_SEP)
	   || match (TOKEN_END))
    {
#ifdef ISPIS
      printf ("Naredba -> eps\n");
#endif
    }
  else
    error ("Ocekivano je while, if, id, begin, eoi, else, ;, end");
}

void
While_naredba ()
{
  if (match (TOKEN_WHILE))
    {
#ifdef ISPIS
      printf ("While_naredba -> while ( Uslov ) do Naredba\n");
#endif
      advance ();
      if (!match (TOKEN_OZ))
	error ("Ocekivana je na ulazu (");
      advance ();
      Uslov ();
      if (!match (TOKEN_ZZ))
	error ("Ocekivano je na ulazu )");
      advance ();
      if (!match (TOKEN_DO))
	error ("Ocekivano je do");
      advance ();
      Naredba ();
    }
  else
    error ("Ocekivano je na ulazu while");
}

void
If_naredba ()
{
  if (match (TOKEN_IF))
    {
#ifdef ISPIS
      printf ("If_naredba -> if ( Uslov ) then Naredba else Naredba \n");
#endif
      advance ();
      if (!match (TOKEN_OZ))
	error ("Ocekivano je na ulazu (");
      advance ();
      Uslov ();
      if (!match (TOKEN_ZZ))
	error ("Ocekivano je na ulazu )");
      advance ();
      if (!match (TOKEN_THEN))
	error ("Ocekivano je na ulazu then");
      advance ();
      Naredba ();
      if (!match (TOKEN_ELSE))
	error ("Ocekivano je na ulazu else\n");
      advance ();
      Naredba ();
    }
  else
    error ("Ocekivano je na ulazu if");
}


void
Naredba_dodele ()
{
  if (match (TOKEN_ID))
    {
#ifdef ISPIS
      printf ("Naredba_dodele -> id := broj\n");
#endif
      advance ();
      if (!match (TOKEN_DODELA))
	error ("Ocekivano je na ulazu :=");
      advance ();
      if (!match (TOKEN_BROJ))
	error ("Ocekivan je na ulazu broj");
      advance ();
    }
  else
    error ("Ocekivano je na ulazu id");
}


void
Blok_naredba ()
{
  if (match (TOKEN_BEGIN))
    {
#ifdef ISPIS
      printf ("Blok_naredba -> begin Niz_naredbi end \n");
#endif
      advance ();
      Niz_naredbi ();
      if (!match (TOKEN_END))
	error ("Ocekivano je na ulazu end");
      advance ();
    }
  else
    error ("Ocekivano je na ulazu begin");
}

void
Niz_naredbi ()
{
  if (match (TOKEN_WHILE) || match (TOKEN_IF) || match (TOKEN_ID) ||
      match (TOKEN_BEGIN) || match (TOKEN_SEP) || match (TOKEN_END))
    {
#ifdef ISPIS
      printf ("Niz_naredbi -> Naredba Niz_naredbi'\n");
#endif
      Naredba ();
      Niz_naredbiP ();
    }
  else
    error ("Ocekivano je na ulazu while, if, id, begin, ;, end");
}

void
Niz_naredbiP ()
{
  if (match (TOKEN_SEP))
    {
#ifdef ISPIS
      printf ("Niz_naredbi' -> ; Naredba Niz_naredbi'\n");
#endif
      advance ();
      Naredba ();
      Niz_naredbiP ();
    }
  else if (match (TOKEN_END))
    {
#ifdef ISPIS
      printf ("Niz_naredbi' -> eps\n");
#endif
    }
  else
    error ("Ocekivano je na ulazu ; ili end");
}

void
Uslov ()
{
  if (match (TOKEN_ID))
    {
#ifdef ISPIS
      printf ("Uslov -> id rel_op broj\n");
#endif
      advance ();
      if (!match (TOKEN_REL_OP))
	error ("Ocekivan je na ulazu relacioni operator");
      advance ();
      if (!match (TOKEN_BROJ))
	error ("Ocekivano je na ulazu broj");
      advance ();
    }
  else
    error ("Ocekivano je na ulazu id");
}

int
main ()
{
  advance ();
  Naredba ();
  if (!match (TOKEN_EOI))
    error ("Visak karaktera na ulazu");
  return 0;
}
